package javax.security.auth.kerberos;

import sun.security.krb5.KerberosSecrets;
import java.util.Objects;
import sun.security.krb5.EncryptionKey;
import sun.security.krb5.RealmException;
import sun.security.krb5.PrincipalName;
import java.security.AccessControlException;
import java.io.File;

public final class KeyTab
{
    private final File file;
    private final KerberosPrincipal princ;
    private final boolean bound;
    
    private KeyTab(KerberosPrincipal princ, File file, boolean bound) {
        this.princ = princ;
        this.file = file;
        this.bound = bound;
    }
    
    public static KeyTab getInstance(File file) {
        if (file == null) {
            throw new NullPointerException("file must be non null");
        }
        return new KeyTab(null, file, true);
    }
    
    public static KeyTab getUnboundInstance(File file) {
        if (file == null) {
            throw new NullPointerException("file must be non null");
        }
        return new KeyTab(null, file, false);
    }
    
    public static KeyTab getInstance(KerberosPrincipal kerberosPrincipal, File file) {
        if (kerberosPrincipal == null) {
            throw new NullPointerException("princ must be non null");
        }
        if (file == null) {
            throw new NullPointerException("file must be non null");
        }
        return new KeyTab(kerberosPrincipal, file, true);
    }
    
    public static KeyTab getInstance() {
        return new KeyTab(null, null, true);
    }
    
    public static KeyTab getUnboundInstance() {
        return new KeyTab(null, null, false);
    }
    
    public static KeyTab getInstance(KerberosPrincipal kerberosPrincipal) {
        if (kerberosPrincipal == null) {
            throw new NullPointerException("princ must be non null");
        }
        return new KeyTab(kerberosPrincipal, null, true);
    }
    
    sun.security.krb5.internal.ktab.KeyTab takeSnapshot() {
        try {
            return sun.security.krb5.internal.ktab.KeyTab.getInstance(this.file);
        }
        catch (AccessControlException ex) {
            if (this.file != null) {
                throw ex;
            }
            AccessControlException ex2 = new AccessControlException("Access to default keytab denied (modified exception)");
            ex2.setStackTrace(ex.getStackTrace());
            throw ex2;
        }
    }
    
    public KerberosKey[] getKeys(KerberosPrincipal kerberosPrincipal) {
        try {
            if (this.princ != null && !kerberosPrincipal.equals(this.princ)) {
                return new KerberosKey[0];
            }
            EncryptionKey[] serviceKeys = this.takeSnapshot().readServiceKeys(new PrincipalName(kerberosPrincipal.getName()));
            KerberosKey[] array = new KerberosKey[serviceKeys.length];
            for (int i = 0; i < array.length; ++i) {
                Integer keyVersionNumber = serviceKeys[i].getKeyVersionNumber();
                array[i] = new KerberosKey(kerberosPrincipal, serviceKeys[i].getBytes(), serviceKeys[i].getEType(), (keyVersionNumber == null) ? 0 : ((int)keyVersionNumber));
                serviceKeys[i].destroy();
            }
            return array;
        }
        catch (RealmException ex) {
            return new KerberosKey[0];
        }
    }
    
    EncryptionKey[] getEncryptionKeys(PrincipalName service) {
        return this.takeSnapshot().readServiceKeys(service);
    }
    
    public boolean exists() {
        return !this.takeSnapshot().isMissing();
    }
    
    @Override
    public String toString() {
        String s = (this.file == null) ? "Default keytab" : this.file.toString();
        if (!this.bound) {
            return s;
        }
        if (this.princ == null) {
            return s + " for someone";
        }
        return s + " for " + this.princ;
    }
    
    @Override
    public int hashCode() {
        return Objects.hash(new Object[] { this.file, this.princ, Boolean.valueOf(this.bound) });
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof KeyTab)) {
            return false;
        }
        KeyTab keyTab = (KeyTab)o;
        return Objects.equals(keyTab.princ, this.princ) && Objects.equals(keyTab.file, this.file) && this.bound == keyTab.bound;
    }
    
    public KerberosPrincipal getPrincipal() {
        return this.princ;
    }
    
    public boolean isBound() {
        return this.bound;
    }
    
    static {
        KerberosSecrets.setJavaxSecurityAuthKerberosAccess(new JavaxSecurityAuthKerberosAccessImpl());
    }
}
